@@ -1,4 +1,4 @@
 from covid_data import db as db  # noqa: F401
 from covid_data import types as types  # noqa: F401
+from covid_data.db import queries as queries  # noqa: F401
 from covid_data.utils import places as places_utils  # noqa: F401
-from db import queries as queries  # noqa: F401
